__all__ = ()

from ..field_parsers import default_entity_parser_factory
from ..field_putters import nullable_entity_optional_putter_factory
from ..field_validators import default_entity_validator

from .message_role_subscription import MessageRoleSubscription

# role_subscription

parse_role_subscription = default_entity_parser_factory(
    'role_subscription_data', MessageRoleSubscription, None
)
put_role_subscription_into = nullable_entity_optional_putter_factory(
    'role_subscription_data', MessageRoleSubscription
)
validate_role_subscription = default_entity_validator(
    'role_subscription', MessageRoleSubscription, None
)



__all__ = ()

from ....permission import Permission
from ....permission.constants import PERMISSION_ALLOW_KEY


def parse_allow(data):
    """
    Parses the `allow` field of ``PermissionOverwrite`` out of the given data.
    
    Parameters
    ----------
    data : `dict` of (`str`, `Any`) items
        Permission overwrite data.
    
    Returns
    -------
    allow : ``Permission``
    """
    return Permission(data[PERMISSION_ALLOW_KEY])


def validate_allow(allow):
    """
    Validates `allow` field of ``PermissionOverwrite``.
    
    Parameters
    ----------
    allow : `None`, ``Permission``, `int`
        The permission overwrite's allowed permission's value.
    
    Returns
    -------
    allow : ``Permission``
    
    Raises
    ------
    TypeError
        - If `allow` is not `None`, `int`, ``Permission``.
    """
    if allow is None:
        allow = Permission()
    
    elif isinstance(allow, Permission):
        pass
    
    elif isinstance(allow, int):
        allow = Permission(allow)
    
    else:
        raise TypeError(
            f'`allow` can be `None`, `{Permission.__name__}`, `int`, got {allow.__class__.__name__}; {allow!r}.'
        )
    
    return allow


def put_allow_into(allow, data, defaults):
    """
    Puts the `allow` field of ``PermissionOverwrite`` to the given data.
    
    Parameters
    ----------
    allow : ``Permission``
        The permission overwrite's allowed permission's value.
    data : `dict` of (`str`, `Any`) items
        Json serializable dictionary.
    defaults : `bool`
        Whether default values should be included as well.
    
    Returns
    -------
    data : `dict` of (`str`, `Any`) items
    """
    data[PERMISSION_ALLOW_KEY] = format(allow, 'd')
    
    return data



from .attachment import *
from .message_activity import *
from .message_role_subscription import *

from .cross_mention import *
from .fields import *
from .flags import *
from .message import *
from .message_application import *
from .message_interaction import *
from .preinstanced import *
from .utils import *


__all__ = (
    *attachment.__all__,
    *message_activity.__all__,
    *message_role_subscription.__all__,
    
    *cross_mention.__all__,
    *fields.__all__,
    *flags.__all__,
    *message.__all__,
    *message_application.__all__,
    *message_interaction.__all__,
    *preinstanced.__all__,
    *utils.__all__,
)
